'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fusionCore = require('fusion-core');
var bodyParser = _interopDefault(require('koa-bodyparser'));
var reactRouter = require('react-router');

const HTTPRouterToken = fusionCore.createToken('HTTPRouter');
const HTTPHandlersToken = fusionCore.createToken('HTTPHandlersToken');
const BodyParserOptionsToken = fusionCore.createToken('BodyParserOptionsToken');

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-env node */
function getHttpHandler(currentPath, method, paths, handlers) {
  const {
    path,
    params = {},
    isExact
  } = reactRouter.matchPath(currentPath, {
    path: paths
  }) || {};
  const handler = handlers[path] && handlers[path][method];

  if (path && handler && isExact) {
    if (typeof handler !== 'function') {
      throw new Error(`Missing/incorrect handler registered to ${method} of ${path}.`);
    }

    return [handler, params];
  }

  return [];
}
const plugin = true && fusionCore.createPlugin({
  deps: {
    bodyParserOptions: BodyParserOptionsToken.optional,
    handlers: HTTPHandlersToken
  },
  provides: ({
    handlers,
    bodyParserOptions
  }) => {
    if (!handlers || typeof handlers !== 'object') {
      throw new Error('Missing/incorrect handlers registered to HTTPHandlersToken');
    }

    const parseBody = bodyParser(bodyParserOptions);
    const paths = Object.keys(handlers);
    const from = fusionCore.memoize(async ctx => {
      const {
        path,
        method,
        query
      } = ctx;
      const [handler, params] = getHttpHandler(path, method, paths, handlers);

      if (typeof handler !== 'function') {
        return null;
      }

      await parseBody(ctx, () => Promise.resolve());
      return () => handler(_objectSpread({}, params, {}, query, {}, ctx.request.body), ctx);
    });
    return {
      from
    };
  },
  middleware: (deps, service) => async (ctx, next) => {
    try {
      const handler = await service.from(ctx);

      if (typeof handler === 'function') {
        ctx.body = await handler();
      }
    } catch (error) {
      ctx.body = {
        status: 'failure',
        data: {
          code: error.code,
          message: error.message,
          stack: error.stack,
          meta: error.meta
        }
      };
    }

    return next();
  }
});

var index = plugin;

exports.default = index;
exports.BodyParserOptionsToken = BodyParserOptionsToken;
exports.HTTPHandlersToken = HTTPHandlersToken;
exports.HTTPRouterToken = HTTPRouterToken;
